!-------------------------------------------------------------------------------
!> module LAND driver
!!
!! @par Description
!!          Land module driver
!!
!! @author Team SCALE
!! @li      2013-08-31 (T.Yamaura)  [new]
!!
!<
!-------------------------------------------------------------------------------
module mod_land
  !-----------------------------------------------------------------------------
  !
  !++ used modules
  !
  use mod_precision
  use mod_stdio
  use mod_prof
  use mod_grid_index
  !-----------------------------------------------------------------------------
  implicit none
  private
  !-----------------------------------------------------------------------------
  !
  !++ Public procedure
  !
  public :: LAND_setup
  public :: LAND_step

  !-----------------------------------------------------------------------------
  !
  !++ Public parameters & variables
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private procedure
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private parameters & variables
  !
  !-----------------------------------------------------------------------------
contains
  !-----------------------------------------------------------------------------
  !> Setup
  subroutine LAND_setup
    use mod_land_vars, only: &
       sw_phy => LAND_sw_phy,  &
       LAND_vars_setup,        &
       LAND_vars_restart_read
    use mod_land_phy_bucket, only: &
       LAND_PHY_driver_setup
    implicit none
    !---------------------------------------------------------------------------

    call LAND_vars_setup

    call LAND_vars_restart_read

    if ( sw_phy ) call LAND_PHY_driver_setup

    return
  end subroutine LAND_setup

  !-----------------------------------------------------------------------------
  !> Land step
  subroutine LAND_step
    use mod_land_vars, only: &
       sw_phy => LAND_sw_phy, &
       LAND_vars_history
    use mod_land_phy_bucket, only: &
       LAND_PHY_driver_first, &
       LAND_PHY_driver_final
    implicit none
    !---------------------------------------------------------------------------

    !########## Physics First ##########
    call PROF_rapstart('LND Physics')
    if ( sw_phy ) then
       call LAND_PHY_driver_first
    endif
    call PROF_rapend  ('LND Physics')

    !########## Physics Final ##########
    call PROF_rapstart('LND Physics')
    if ( sw_phy ) then
       call LAND_PHY_driver_final
    endif
    call PROF_rapend  ('LND Physics')

    !########## History & Monitor ##########
    call PROF_rapstart('LND History')
    call LAND_vars_history
    call PROF_rapend  ('LND History')

    return
  end subroutine LAND_step

end module mod_land
