!-------------------------------------------------------------------------------
!> module ATMOSPHERE / Physics Cloud Microphysics
!!
!! @par Description
!!         Cloud Microphysics
!!
!! @author Team SCALE
!!
!! @par History
!! @li      2013-12-06 (S.Nishizawa) [new]
!!
!<
! Warning: This file was generated from atmos/mod_atmos_phy_mp.F90.erb.
!          Do not edit this file.
!-------------------------------------------------------------------------------
module mod_atmos_phy_mp
  !-----------------------------------------------------------------------------
  !
  !++ used modules
  !
  use mod_precision
  use mod_stdio
  use mod_prof
  use mod_grid_index
  use mod_tracer
  !-----------------------------------------------------------------------------
  implicit none
  private
  !-----------------------------------------------------------------------------
  !
  !++ Public procedure
  !
  public :: ATMOS_PHY_MP_setup

  !-----------------------------------------------------------------------------
  !
  !++ Public parameters & variables
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private procedure
  !
  !-----------------------------------------------------------------------------
  !
  !++ Private parameters & variables
  !
  abstract interface
     subroutine mp( &
          DENS, &
          MOMZ, &
          MOMX, &
          MOMY, &
          RHOT, &
          QTRC  )
       use mod_precision
       use mod_grid_index
       use mod_tracer
       real(RP), intent(inout) :: DENS(KA,IA,JA)
       real(RP), intent(inout) :: MOMZ(KA,IA,JA)
       real(RP), intent(inout) :: MOMX(KA,IA,JA)
       real(RP), intent(inout) :: MOMY(KA,IA,JA)
       real(RP), intent(inout) :: RHOT(KA,IA,JA)
       real(RP), intent(inout) :: QTRC(KA,IA,JA,QA)
     end subroutine mp
     subroutine cf( &
          cldfrac, &
          QTRC     )
       use mod_precision
       use mod_grid_index
       use mod_tracer
       real(RP), intent(out) :: cldfrac(KA,IA,JA)
       real(RP), intent(in)  :: QTRC   (KA,IA,JA,QA)
     end subroutine cf
     subroutine er( &
          Re,    &
          QTRC0, &
          DENS0  )
       use mod_precision
       use mod_grid_index
       use mod_tracer
       real(RP), intent(out) :: Re   (KA,IA,JA,MP_QA) ! effective radius
       real(RP), intent(in)  :: QTRC0(KA,IA,JA,QA)    ! tracer mass concentration [kg/kg]
       real(RP), intent(in)  :: DENS0(KA,IA,JA)       ! density
     end subroutine er
     subroutine mr( &
          Qe,    &
          QTRC0  )
       use mod_precision
       use mod_grid_index
       use mod_tracer
       real(RP), intent(out) :: Qe   (KA,IA,JA,MP_QA) ! mixing ratio of each cateory [kg/kg]
       real(RP), intent(in)  :: QTRC0(KA,IA,JA,QA)    ! tracer mass concentration [kg/kg]
     end subroutine mr
  end interface
  procedure(mp), pointer, public :: ATMOS_PHY_MP => NULL()
  procedure(cf), pointer, public :: ATMOS_PHY_MP_CloudFraction => NULL()
  procedure(er), pointer, public :: ATMOS_PHY_MP_EffectiveRadius => NULL()
  procedure(mr), pointer, public :: ATMOS_PHY_MP_MixingRatio => NULL()

  real(RP), pointer, public :: ATMOS_PHY_MP_DENS(:)

  !-----------------------------------------------------------------------------
contains
  !-----------------------------------------------------------------------------
  !> Setup Cloud Microphysics
  !-----------------------------------------------------------------------------
  subroutine ATMOS_PHY_MP_setup( MP_TYPE )
    use mod_process, only: &
       PRC_MPIstop
#define EXTM(pre, name, post) pre ## name ## post
#define NAME(pre, name, post) EXTM(pre, name, post)
#ifdef MP
    use NAME(mod_atmos_phy_mp_, MP,), only: &
       NAME(ATMOS_PHY_MP_, MP, _setup), &
       NAME(ATMOS_PHY_MP_, MP,), &
       NAME(ATMOS_PHY_MP_, MP, _CloudFraction), &
       NAME(ATMOS_PHY_MP_, MP, _EffectiveRadius), &
       NAME(ATMOS_PHY_MP_, MP, _MixingRatio), &
       NAME(ATMOS_PHY_MP_, MP, _DENS) => ATMOS_PHY_MP_DENS)
#else
    use mod_atmos_phy_mp_dry, only: &
       ATMOS_PHY_MP_dry_setup, &
       ATMOS_PHY_MP_dry, &
       ATMOS_PHY_MP_dry_CloudFraction, &
       ATMOS_PHY_MP_dry_EffectiveRadius, &
       ATMOS_PHY_MP_dry_MixingRatio, &
       ATMOS_PHY_MP_dry_DENS => ATMOS_PHY_MP_DENS
    use mod_atmos_phy_mp_kessler, only: &
       ATMOS_PHY_MP_kessler_setup, &
       ATMOS_PHY_MP_kessler, &
       ATMOS_PHY_MP_kessler_CloudFraction, &
       ATMOS_PHY_MP_kessler_EffectiveRadius, &
       ATMOS_PHY_MP_kessler_MixingRatio, &
       ATMOS_PHY_MP_kessler_DENS => ATMOS_PHY_MP_DENS
    use mod_atmos_phy_mp_tomita08, only: &
       ATMOS_PHY_MP_tomita08_setup, &
       ATMOS_PHY_MP_tomita08, &
       ATMOS_PHY_MP_tomita08_CloudFraction, &
       ATMOS_PHY_MP_tomita08_EffectiveRadius, &
       ATMOS_PHY_MP_tomita08_MixingRatio, &
       ATMOS_PHY_MP_tomita08_DENS => ATMOS_PHY_MP_DENS
    use mod_atmos_phy_mp_sn13, only: &
       ATMOS_PHY_MP_sn13_setup, &
       ATMOS_PHY_MP_sn13, &
       ATMOS_PHY_MP_sn13_CloudFraction, &
       ATMOS_PHY_MP_sn13_EffectiveRadius, &
       ATMOS_PHY_MP_sn13_MixingRatio, &
       ATMOS_PHY_MP_sn13_DENS => ATMOS_PHY_MP_DENS
    use mod_atmos_phy_mp_suzuki10, only: &
       ATMOS_PHY_MP_suzuki10_setup, &
       ATMOS_PHY_MP_suzuki10, &
       ATMOS_PHY_MP_suzuki10_CloudFraction, &
       ATMOS_PHY_MP_suzuki10_EffectiveRadius, &
       ATMOS_PHY_MP_suzuki10_MixingRatio, &
       ATMOS_PHY_MP_suzuki10_DENS => ATMOS_PHY_MP_DENS
#endif
    implicit none

    character(len=H_SHORT), intent(in) :: MP_TYPE
    !---------------------------------------------------------------------------

    select case ( MP_TYPE )
    case ( 'DRY' )
       call ATMOS_PHY_MP_dry_setup( MP_TYPE )
       ATMOS_PHY_MP => ATMOS_PHY_MP_dry
       ATMOS_PHY_MP_CloudFraction => ATMOS_PHY_MP_dry_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_dry_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio => ATMOS_PHY_MP_dry_MixingRatio
       ATMOS_PHY_MP_DENS => ATMOS_PHY_MP_dry_DENS
    case ( 'KESSLER' )
       call ATMOS_PHY_MP_kessler_setup( MP_TYPE )
       ATMOS_PHY_MP => ATMOS_PHY_MP_kessler
       ATMOS_PHY_MP_CloudFraction => ATMOS_PHY_MP_kessler_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_kessler_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio => ATMOS_PHY_MP_kessler_MixingRatio
       ATMOS_PHY_MP_DENS => ATMOS_PHY_MP_kessler_DENS
    case ( 'TOMITA08' )
       call ATMOS_PHY_MP_tomita08_setup( MP_TYPE )
       ATMOS_PHY_MP => ATMOS_PHY_MP_tomita08
       ATMOS_PHY_MP_CloudFraction => ATMOS_PHY_MP_tomita08_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_tomita08_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio => ATMOS_PHY_MP_tomita08_MixingRatio
       ATMOS_PHY_MP_DENS => ATMOS_PHY_MP_tomita08_DENS
    case ( 'SN13' )
       call ATMOS_PHY_MP_sn13_setup( MP_TYPE )
       ATMOS_PHY_MP => ATMOS_PHY_MP_sn13
       ATMOS_PHY_MP_CloudFraction => ATMOS_PHY_MP_sn13_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_sn13_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio => ATMOS_PHY_MP_sn13_MixingRatio
       ATMOS_PHY_MP_DENS => ATMOS_PHY_MP_sn13_DENS
    case ( 'SUZUKI10' )
       call ATMOS_PHY_MP_suzuki10_setup( MP_TYPE )
       ATMOS_PHY_MP => ATMOS_PHY_MP_suzuki10
       ATMOS_PHY_MP_CloudFraction => ATMOS_PHY_MP_suzuki10_CloudFraction
       ATMOS_PHY_MP_EffectiveRadius => ATMOS_PHY_MP_suzuki10_EffectiveRadius
       ATMOS_PHY_MP_MixingRatio => ATMOS_PHY_MP_suzuki10_MixingRatio
       ATMOS_PHY_MP_DENS => ATMOS_PHY_MP_suzuki10_DENS
    end select

    return
  end subroutine ATMOS_PHY_MP_setup

end module mod_atmos_phy_mp
